Class: OAuth::CLI::HelpCommand

Inherits:
BaseCommand show all
Defined in:
lib/oauth/cli/help_command.rb

Instance Method Summary collapse

Methods inherited from BaseCommand

#initialize, #required_options

Constructor Details

This class inherits a constructor from OAuth::CLI::BaseCommand

Instance Method Details

#runObject



6
7
8
9
10
11
12
13
14
15
16
17
18
19
20
21
22
23
# File 'lib/oauth/cli/help_command.rb', line 6

def run
  puts <<-EOT
  Usage: oauth COMMAND [ARGS]

  Available oauth commands are:
    a, authorize  Obtain an access token and secret for a user
    q, query      Query a protected resource
    s, sign       Generate an OAuth signature

  In addition to those, there are:
    v, version    Displays the current version of the library (or --version, -v)
    h, help       Displays this help (or --help, -h)

  Tip: All commands can be run without args for specific help.


  EOT
end